function [A, submod_fval, h] = submod_oracle_group(g, lambda, sbd_ctx, par)

% The submodular oracle using max-flow.
%     A = \argmax_A <g, 1_A> - \lambda F(A)
% Input:
%   g: weights on the nodes
%   lambda: weight on F(A)
%   sbd_ctx: context for submodular optimization, which includes the group graph
%   par: the parameters of the experiment
%
% Output:
%   A: a 0/1 vector representing the optimal set A
%   submod_fval: F(A) for the optimal A
%   h: the maximum value of <g, 1_A> - \lambda F(A)

  [num_group, num_var] = size(sbd_ctx.G_by_V);
  [ori_ilist, ori_jlist, ~] = find(sbd_ctx.G_by_V);
  g_plus_v = num_var + num_group;
  
  % Add arcs from source to all groups, and from all nodes to sink
  % whole matrix is group + vertex
  if par.polar_par.use_maxflow_bk
    bk_mat = sparse(ori_ilist, ori_jlist + num_group, ...
                      9e9*ones(length(ori_ilist), 1), g_plus_v, g_plus_v);
    % First source to groups, then vertices to sink
    st_mat = [[lambda*ones(num_group,1); zeros(num_var,1)], ...
               [zeros(num_group, 1); g]];

    [flow_bk, cut] = maxflow(bk_mat, sparse(st_mat));
    h = sum(g) - flow_bk;
%     fprintf('BK min-cut: flow_bk = %g, time = %g\n', flow_bk, cputime - tstart);
  end
  
  if par.polar_par.use_maxflow_darwin
    edgeList = [[-1*ones(num_group,1), (0:num_group-1)', lambda*ones(num_group,1)]
                [ori_ilist-1, ori_jlist+num_group-1, 9e9*ones(length(ori_ilist),1)]
                [(num_group:g_plus_v-1)', -1*ones(num_var,1), g]];
    options = struct('verbose', 0);

    [flow_dw, cut] = mexMaxFlow(edgeList, options);
    h = sum(g) - flow_dw;
%     fprintf('Darwin min-cut: flow_bk = %g, time = %g\n', flow_dw, cputime-tstart);
  end
  
  if par.polar_par.use_maxflow_bk && par.polar_par.use_maxflow_darwin
    assert(abs(flow_bk - flow_dw)/max(abs([flow_bk,flow_dw])) < 1e-2);
  end
  
  A = cut(num_group+1:g_plus_v) > 0.5;  
  submod_fval = sum(cut(1:num_group) > 0.5);  
  
end
