%  [X, fval] = prox_op_sbd(z, lambda, sbd_ctx, par): given z, solves 
%        (*) min_{x} <x, -z> + lambda*reg(x) + 0.5*||x||^2
%           where reg is the norm induced by a submodular function
%
%        It is equivalent to
%        (**)  min_{x} 0.5*||x - z||^2 + lambda*reg(x) 
%  Input:
%    z: the point at which the proximal operator is called
%    lambda: the weight on the norm regularizer
%    sbd_ctx: the submodular context (for graph sparsity, it contains
%         G_by_V: a sparse matrix of group-by-vertix
%         V_by_G: the transpose of G_by_V
%         They are used for submodular minimization.
%    par: the parameters of the experiment
%  Output:
%    X: the proximal projection
%    fval: the value of the proximal objective:
%      fval.reg: value of the regularizer at the solution X (not multiplied by lambda)
%      fval.obj_lin: value of the proximal operator objective (*)
%      fval.obj_MY: value of the proximal operator objective (**) as Moreau-Yosida regularizer

function [X, fval] = prox_op_sbd(z, lambda, sbd_ctx, par)
  
  d = length(z);
  p = par.exp_par.p;
  if isinf(p), q = 1; else q = p/(p-1); end
  supp = abs(z) > 1e-7;
  X = zeros(d,1);
  
  nnz = sum(supp);
  if nnz < d
    if nnz == 0 
      fval = struct('reg', 0, 'obj_lin', 0, 'obj_MY', 0);
      return;
    end
    sbd_ctx = par.sbd_restrict(supp, sbd_ctx, par);
    [X(supp), fval] = prox_op_sbd(z(supp), lambda, sbd_ctx, par);    
    return;
  end
  
  fV = par.sbd_fun(true(d,1), sbd_ctx, par);  % Compute F(V)
%   fprintf('double check: d = %d, fV = %g, num_group=%g\n', d, fV, ...
%             size(sbd_ctx.G_by_V, 1));
  t = abs(z);
  if isinf(p)
    [~,idx] = max(t);
    t = zeros(d, 1);
    t(idx) = fV;
  else
    t = (t / max(t)).^p;
    t = t / sum(t) * fV;  % line 5 of Algorithm 2 of Obo and Bach
  end
  
  A = par.sbd_oracle(t, 1, sbd_ctx, par);   % solve min_A F(A) - t(A)
  if all(~A)
    A = ~A;
  end
  euc_norm_sq_z = sum(z.^2);
  if sum(A) == d % do line 8 of Algorithm 2 of Obo and Bach
    t = abs(z);
    max_t = max(t);
    if isinf(p)
      a = max_t * fV;
    else
      a = max_t*sum((t/max_t).^p)^(1/p) * fV^(1/q);  % line 8 of Alg 2
    end
    r_on_a = max(0, 1 - lambda*a / euc_norm_sq_z);  % = 1 if lambda = 0.
    r = a*r_on_a;
    X = r_on_a*z;
    fval = struct('reg', r, ...
                  'obj_lin', r_on_a*(r_on_a-2)*euc_norm_sq_z/2 + lambda*r, ...
                  'obj_MY', (1-r_on_a)^2*euc_norm_sq_z/2 + lambda*r);
  else % Decompose and do line 10 of Algorithm 2 of Obo and Bach
%     fprintf('Splitting: %d -> %d + %d\n', length(A), sum(A), sum(~A));
    sbd_ctx_1 = par.sbd_restrict(A, sbd_ctx, par);  % Restriction
    [X(A), fval1] = prox_op_sbd(z(A), lambda, sbd_ctx_1, par);

    sbd_ctx_2 = par.sbd_restrict(~A, sbd_ctx, par); % Contraction
    [X(~A), fval2] = prox_op_sbd(z(~A), lambda, sbd_ctx_2, par);

    fval = struct('reg', fval1.reg + fval2.reg, ...
                  'obj_lin', fval1.obj_lin + fval2.obj_lin, ...
                  'obj_MY', fval1.obj_MY + fval2.obj_MY);
  end
end
