function f = kappa(U, V, par)
% Implements the dual norm:
%
% max_{eta \in [0, 1]} || D^{-1}_{(1-eta)/(gamma*eta)}  Z ||_tr
%
%   Parameters:
%       Z:  matrix
%       n1: num rows of Z_1
%       n2: num rows of Z_2
% where Lambda = [Z_1; Z_2] is the dual matrix
%       gamma:  needed to define 
%           D_rho = diag(sqrt(1+gamma^2*rho) 1_n1;
%                         sqrt(gamma^2 + 1/rho) 1_n2]
%
% Author: Martha White, University of Alberta, 2011
% updated by Yao-Liang: use the true subgradient in dual_norm_rho
% updated by Xinhua: changed line search in rho to in eta, because the line
%                     search in eta is concave (not just quasi-concave)

  beta = par.prob_par.beta;
  alpha = par.prob_par.alpha;
  m1 = par.prob_par.m1;
  
  W = U*V;
  W1 = W(1:m1,:);
  W2 = W(m1+1:end,:); 
  
  U1 = U(1:m1,:);
  U2 = U(m1+1:end,:); 

  lb = 1e-3;  ub = 1-lb;    eta_init = 0.5;

  param = [];
  param.maxIter = 20;     % max number of iterations
  param.maxLsIter = 20;   % max number of line search steps in each iteration
  param.maxBdl = 30;      % max number of bundles to keep
  param.maxFnCall = 1e6;  % max number of calling the function
  param.tolCon = 1e-5;      % tolerance of constraint satisfaction
  param.tolFun = 1e-5;   % final objective function accuracy parameter
  param.verbose = 0;      % print intermediate steps (doesn't seem to work)

  x0 = 0.3;
  [g_test, err] = gradest(@obj, x0);
  [f, g] = obj(x0);

  [eta, f, iters, ~, flag] = pbm(@obj_mat, eta_init, lb, ub, param);
  f = -f;
  fprintf('pbm: %s\n', msg2str(flag, 'pbm'));
  
    function [f, g] = obj(eta)
      warning off
      [P, S, Q] = lansvd(@(x)Ytimesx(x,eta), ...
                         @(x)Ytranstimesx(x,eta), size(U,1), size(V,2));
      warning on
      
      f = -sum(diag(S));  % diag allows S to be rectangular
      P1 = P(1:m1,:);
      P2 = P(1+m1:end,:);
      VQ = V*Q;
      a = sum(sum(VQ .* (U1'*P1)));
      b = sum(sum(VQ .* (U2'*P2)));
      g = [a/alpha/sqrt(eta); -b/beta/sqrt(1-eta)]/2;
      g = -sum(g);
    end
  
    function [f, g] = obj_mat(eta)
      
      Y = [sqrt(eta)/alpha*W1; sqrt(1-eta)/beta*W2];
      [P, S, Q] = svd(Y, 'econ');
      
      f = -sum(diag(S));  % diag allows S to be rectangular      
      L = P*Q';
      a = sum(sum(W1 .* L(1:m1,:)));
      b = sum(sum(W2 .* L(1+m1:end,:)));
      g = [a/alpha/sqrt(eta); -b/beta/sqrt(1-eta)]/2;
      
      g = -sum(g);
    end
  
    function res = Ytimesx(x,eta)
      Vx = V*x;
      res = [sqrt(eta)/alpha*(U1*Vx)
             sqrt(1-eta)/beta*(U2*Vx)];
    end

    function res = Ytranstimesx(x,eta)
      x1 = x(1:m1);   x2 = x(1+m1:end);
      res = sqrt(eta)/alpha*(V'*(U1'*x1)) + sqrt(1-eta)/beta*(V'*(U2'*x2));
    end
end
