function [sol, obj, res] = solve_LOB(objf, lambda, X0, evalf, opts)
        
  % Partially corrective boosting
  % Solve  Min_X  func(X) + lambda * ||X||_*

  global feval_logistic time_logistic

  [m, n] = size(X0);
  k = opts.init_rank;  % Initialize U and V with rank k

  U = randn(m, k);
  V = randn(k, n);

  res = struct('obj_list', [], ...
               'loss_list', [], ...
               'perf_list', [] ...
               );
  
  msg = 'Running';
  break_flag = 0;
  perf = struct('l1', 0, 'l2', 0);  
  par = opts.ori_par;
  nidx = 0;
  obj = inf;
  
  time_list = [];
  time_local = [];
  if opts.use_local
    time_polar = [];
  else
    time_polar = 0;
  end
  
  feval_logistic = 0;     time_logistic = 0;
  
  t1 = cputime;  
  for i = 1 : opts.max_iter

    k = size(U, 2);

    % Local Search
    if k > 0
      t2 = cputime;
      if opts.use_local
        
        nel = k*(m+n);
        ub = Inf(nel, 1);
        % local search    
%         disp('local search now');
        [UV, obj, ~, ~, msg] = lbfgsb([U(:); V(:)], -ub, ub, @obj_UV, ...
                                                  [], [], opts.lbfgsb_in);
        U = reshape(UV(1:k*m), [m, k]);
        V = reshape(UV(1+k*m:end), [k, n]);
      end
      
      time_local = [time_local; cputime - t2];
      time_list = [time_list; cputime - t1];
      
      if isinf(obj)
        obj = obj_UV([U(:); V(:)]);
      end
      
      loss = obj - 0.5*lambda*(norm(U, 'fro')^2 + norm(V, 'fro')^2);
      res.loss_list = [res.loss_list; loss];    
      res.obj_list = [res.obj_list; obj];
      
      if ~isempty(evalf)
        perf = evalf({U, V'});
        res.perf_list = [res.perf_list; perf];
      end

      % Prune U and V
      norm_U = sqrt(sum(U.*U))';
      norm_V = sqrt(sum(V.*V, 2));
      norm_UV = sqrt(norm_U .* norm_V);
      score = sqrt(norm_U.^2 + norm_V.^2); 
      idx = score > 1e-5;
      nidx = sum(idx);
      
      if nidx < size(U, 2)
        U = U(:, idx);
        V = V(idx, :);
        norm_U = norm_U(idx);
        norm_V = norm_V(idx);
        norm_UV = norm_UV(idx);
      end
%       fprintf('iter=%d, loss = %g, obj=%g, perf=(%g, %g), k=%d, time=%g, ls_time=%g, msg=%s\n', ...
%             i, loss, obj, perf.l1, perf.l2, k, sum(time_list), sum(time_local), msg2str(msg, 'lbfgsb'));
    end

    if i > 1
      if abs(pre_obj-obj) / min(abs([pre_obj, obj])) < opts.rtol
        outputs.msg = 'Stop with small relative change';
        break_flag = 1;
      elseif sum(time_list) > opts.max_time
        outputs.msg = 'Stop by max_time';
        break_flag = 1;
      elseif i == opts.max_iter
        outputs.msg = 'Stop with max iteration';  
        break_flag = 1;
      end
    end
    
    if (opts.use_local && k > 0) || i > 1 
      res.time_list = cumsum(time_list);
      res.time_polar = cumsum(time_polar);
      res.time_local = cumsum(time_local);
      res.msg = msg;
      res.iter = i;
      res.B = (res.obj_list(end) - res.loss_list(end)) / lambda;

      if isfield(opts, 'res_fname')
        save(opts.res_fname, 'res', 'par');
      end
    end
    
    if break_flag == 1, break;    end
        
    pre_obj = obj;
    t1 = cputime;
    if nidx > 0
      sk = 0.5*(norm_U'*norm_U + norm_V'*norm_V); % Xk = U*V;    
    else
      sk = 0; % Xk = zeros(m, n);
    end

    % Get a gradient and compute the polar oracle
    if size(U, 2) > 0
      [~, G] = objf([{U}, {V'}]);    % G is a sparse matrix
    else
      [~, G] = objf([{zeros(m, 1)}, {zeros(n,1)}]); 
    end

    t2 = cputime;
    warning off
    [u, ~, v] = lansvd(@(X)cell_times_mat(G,X), ...
                       @(X)cell_trans_times_mat(G,X), m, n, 1);
    warning on
    time_polar = [time_polar; cputime - t2];
    clear G
    v = -v;

    % 2-D line search
%     disp('2-D line search now');
    [weights, obj, ~, ~, msg] = lbfgsb([1; 0.5], [0; 0], [inf; inf], ...
                                        @obj_ls, [], [], opts.lbfgsb_in);

%     loss = obj - lambda*(sk*weights(1) + weights(2));
    weights = sqrt(weights);
    if nidx > 0
        U = [bsxfun(@times, U, weights(1)*(norm_UV./norm_U)'), weights(2) * u];
        V = [bsxfun(@times, V, weights(1)*norm_UV./norm_V); weights(2) * v']; 
    else
        U = weights(2) * u;
        V = weights(2) * v'; 
    end

  end

  sol = {U, V'};
  outputs.iter = i;

  if opts.verbose
    fprintf(1, '%s\n', outputs.msg);
  end
  
    % local search objective
    function [f, g] = obj_UV(X)
      r = length(X) / (m+n);
      UU = reshape(X(1:r*m), [m, r]);
      VV = reshape(X(1+r*m:end), [r, n]);

      [f, GG] = objf({UU, VV'});
      f = f + 0.5*lambda*norm(X)^2;
      g = [vec(cell_times_mat(GG, VV') + lambda*UU)
          vec(mat_times_cell(UU', GG) +lambda*VV)];   
    end

    % Line search objective
    function [f, g] = obj_ls(x)
      [f, GG] = objf({[sqrt(x(1))*U, sqrt(x(2))*u], [sqrt(x(1))*V', sqrt(x(2))*v]});
      f = f + lambda*(sk*x(1) + x(2));
      g = [sum(sum(U .* cell_times_mat(GG, V')))+lambda*sk
           u'*cell_times_mat(GG, v)+lambda];
    end
end
